import java.util.concurrent.*;
import java.util.List;
import java.util.LinkedList;

public class SampleApp extends BaseApp implements IApp {

    private ExecutorService pool;

    public SampleApp() {
        // Create the thread pool
        pool = Executors.newFixedThreadPool(10);
    }

    public void start() {
        List<Future> futureList = new LinkedList<Future>();

        for (int i = 0; i < 10; i++) {
            ProcessorTask task = new ProcessorTask(String.valueOf(i));
            Future future = pool.submit(task);
            futureList.add(future);
        }

        // You can get the result back from the thread
        for (Future future : futureList) {
            try {
                // Wait with a time out if necessary
                //Object obj = future.get(5, TimeUnit.SECONDS);
                Object obj = future.get();
            } catch (InterruptedException e) {
                System.out.println(e.getMessage());
            } catch (ExecutionException e) {
                System.out.println(e.getMessage());
            }
        }

        pool.shutdown();

    }

    public void shutDown() {
        // Do a graceful shutdown here
        System.out.println("Shutting down the application");

    }


    public static void main(String args[]) {
        IApp app = new SampleApp();
        ShutdownInterceptor shutdownInterceptor = new ShutdownInterceptor(app);
        Runtime.getRuntime().addShutdownHook(shutdownInterceptor);
        app.start();
    }
}
